import sqlite3
import re


def migrate(db, habitlist, events):
    c = db.cursor()
    habits = habit_list_add_ids(c, habitlist)
    repetitions = get_all_repetitions(habits, events)
    for rep in repetitions:
        add_to_database(
            c, rep["id"], rep["timestamp"], 2 if "value" not in rep else rep["value"]
        )


LOOP_RANGE_VALUE_MULTIPLIER = 1000


def get_all_repetitions(habits, events):
    """Return list of all repetitions found of habits in events passed in.
    Parameters:
    habits (list): Collection of habits, with minimum necessary fields description and id.
    events (list): Collection of events, with minimum necessary field end.
    Returns:
    repetitions (list): Collection of events transformed into Loop repetitions.
                        Contains fields id, timestamp, value (for ranges).
    """
    repetitions = []
    for event in events:
        for habit_id in habits.keys():
            reps = tags_to_repetitions(
                habit_id, habits[habit_id], extract_tags(event.text), event.end
            )
            if reps:
                repetitions.extend(reps)

    return repetitions


def extract_tags(text, tagmarker="#"):
    """Return lists of tuples of all event tags found in text.
    Parameters:
    text (str): The text to search through.
    tagmarker (str): Optional character marking beginning of tag, defaults to '#'.
    Returns:
    tags (list): List of tuples in the form [('tag', '3'), ('anothertag', '')].
    """
    string_tags = re.findall(rf"{tagmarker}(\w+)(?:\((\d+)\))?", text)
    tags_with_int_counters = []
    for tag in string_tags:
        tags_with_int_counters.append((tag[0], None if tag[1] == "" else int(tag[1])))
    return tags_with_int_counters


# does not do:
# non-range habits but still #habit(3) number included, adding multiple?
def tags_to_repetitions(habit_id, habit, tags, timestamp):
    """Return a list of all repetitions generated from the tags and habits passed in.
    Parameters:
    habits (list): Collection of habits, with minimum necessary fields description and id.
    tags (list): Collection of tag tuples.
    Returns:
    repetitions (list): Collection of habits for which a corresponding tag has
                        been found. If they correspond that means at the timestamp
                        the habit has been checked in, and a repetition is created.
                        Contains fields id, timestamp, value (for ranges).
    """
    reps = []
    for tag in tags:
        if habit.description in tag[0]:
            repetition = {"id": habit_id, "timestamp": timestamp}
            if tag[1]:
                if habit.type == 1:
                    repetition["value"] = tag[1] * LOOP_RANGE_VALUE_MULTIPLIER
            reps.append(repetition)
    return reps


# TODO possibly just get rid of this entirely
def habit_list_add_ids(c, habitlist):
    """Return the collection of habits with their sqlite id added.
    Parameters:
    c (sqlite.db.cursor): SQL cursor of database to query.
    habitlist (list[Habit]): Full habit collection to return a simplified view of.
    Returns:
    habit_id_dict (dict[Habit]): The habit collection as a dict with the keys
                                 consisting of the habit's sqlite database ID.
    """
    with_id = {}
    for h in habitlist:
        sql_id = fetch_habit_id(c, h.uuid)
        with_id[sql_id] = h

    return with_id


def fetch_habit_id(c, uuid):
    """Return sqlite internal id for habit with uuid.
    Parameters:
    c (sqlite.db.cursor): SQL cursor of database to query.
    uuid (str): Unique id of habit to query for.
    Returns:
    id (int): SQLite internal id for habit queried for.
    """
    c.execute("select id from Habits where uuid = ?", ([uuid]))
    id = c.fetchone()
    if id is not None:
        return id[0]


def add_to_database(cursor, habit_id, timestamp, value=2):
    try:
        cursor.execute(
            """
            INSERT INTO
            Repetitions(id, habit, timestamp, value)
            VALUES (NULL, ?, ?, ?)
            """,
            (habit_id, timestamp, value),
        )
    except sqlite3.IntegrityError:
        # TODO better error handling
        print(f"fail to register {habit_id}: timestamp {timestamp} not unique")
